@@ -1,4 +1,4 @@
-/*	$OpenBSD: bgpd.h,v 1.486 2024/02/19 10:15:35 job Exp $ */
+/*	$OpenBSD: bgpd.h,v 1.493 2024/05/18 11:17:30 jsg Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -35,7 +35,7 @@
 #define	BGP_VERSION			4
 #define	BGP_PORT			179
 #define	RTR_PORT			323
-#define	PEER_DESCR_LEN			32
+#define	PEER_DESCR_LEN			64
 #define	REASON_LEN			256	/* includes NUL terminator */
 #define	PFTABLE_LEN			32
 #define	ROUTELABEL_LEN			32
@@ -51,6 +51,7 @@
 #define	RT_BUF_SIZE			16384
 #define	MAX_RTSOCK_BUF			(2 * 1024 * 1024)
 #define	MAX_COMM_MATCH			3
+#define	MAX_ASPA_SPAS_COUNT		10000
 
 #define	BGPD_OPT_VERBOSE		0x0001
 #define	BGPD_OPT_VERBOSE2		0x0002
@@ -418,6 +419,17 @@ struct capabilities {
 	int8_t	policy;			/* Open Policy, RFC 9234, 2 = enforce */
 };
 
+enum capa_codes {
+	CAPA_NONE = 0,
+	CAPA_MP = 1,
+	CAPA_REFRESH = 2,
+	CAPA_ROLE = 9,
+	CAPA_RESTART = 64,
+	CAPA_AS4BYTE = 65,
+	CAPA_ADD_PATH = 69,
+	CAPA_ENHANCED_RR = 70,
+};
+
 /* flags for RFC 4724 - graceful restart */
 #define	CAPA_GR_PRESENT		0x01
 #define	CAPA_GR_RESTART		0x02
@@ -431,6 +443,9 @@ struct capabilities {
 #define	CAPA_AP_RECV		0x01
 #define	CAPA_AP_SEND		0x02
 #define	CAPA_AP_BIDIR		0x03
+#define	CAPA_AP_MASK		0x0f
+#define	CAPA_AP_RECV_ENFORCE	0x10	/* internal only */
+#define	CAPA_AP_SEND_ENFORCE	0x20	/* internal only */
 
 /* values for RFC 9234 - BGP Open Policy */
 #define CAPA_ROLE_PROVIDER	0x00
@@ -474,7 +489,6 @@ struct peer_config {
 	uint8_t			 distance;	/* 1 = direct, >1 = multihop */
 	uint8_t			 passive;
 	uint8_t			 down;
-	uint8_t			 announce_capa;
 	uint8_t			 reflector_client;
 	uint8_t			 ttlsec;	/* TTL security hack */
 	uint8_t			 flags;
@@ -1466,7 +1480,6 @@ void		 log_peer_warnx(const struct peer_config *, const char *, ...)
 			__attribute__((__format__ (printf, 2, 3)));
 
 /* mrt.c */
-void		 mrt_clear_seq(void);
 void		 mrt_write(struct mrt *);
 void		 mrt_clean(struct mrt *);
 void		 mrt_init(struct imsgbuf *, struct imsgbuf *);
@@ -1477,10 +1490,6 @@ struct mrt	*mrt_get(struct mrt_head *, struct mrt *);
 void		 mrt_mergeconfig(struct mrt_head *, struct mrt_head *);
 
 /* name2id.c */
-uint16_t	 rib_name2id(const char *);
-const char	*rib_id2name(uint16_t);
-void		 rib_unref(uint16_t);
-void		 rib_ref(uint16_t);
 uint16_t	 rtlabel_name2id(const char *);
 const char	*rtlabel_id2name(uint16_t);
 void		 rtlabel_unref(uint16_t);
@@ -1544,6 +1553,7 @@ int	trie_equal(struct trie_head *, struct trie_head *);
 time_t			 getmonotime(void);
 
 /* util.c */
+char		*ibuf_get_string(struct ibuf *, size_t);
 const char	*log_addr(const struct bgpd_addr *);
 const char	*log_in6addr(const struct in6_addr *);
 const char	*log_sockaddr(struct sockaddr *, socklen_t);
@@ -1556,6 +1566,7 @@ const char	*log_roa(struct roa *);
 const char	*log_aspa(struct aspa_set *);
 const char	*log_rtr_error(enum rtr_error);
 const char	*log_policy(enum role);
+const char	*log_capability(uint8_t);
 int		 aspath_asprint(char **, struct ibuf *);
 uint32_t	 aspath_extract(const void *, int);
 int		 aspath_verify(struct ibuf *, int, int);
